@@ -23,6 +23,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>	/* FIXME: libraries shouldn't do this */
+#include <errno.h>
 #include <sys/stat.h>
 
 #include <fcntl.h>
@@ -39,12 +40,12 @@
 #define S_IROTH 0
 #define S_IXGRP 0
 #define S_IXOTH 0
-#define _POSIX_PATH_MAX MAX_PATH
 #endif
 #define DEFFILEMOD (S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH) /* 0644 */
 #define DEFXFILEMOD (DEFFILEMOD | S_IXGRP | S_IXUSR | S_IXOTH) /* 0755 */
 
 /* Get some file-info. (size and lastmod) */
+/* lastmod needs to have at least 21 bytes */
 static int get_fileinfo(const char *name, char *lastmod)
 {
 	struct stat stats;
@@ -62,7 +63,7 @@ static int get_fileinfo(const char *name, char *lastmod)
 
 
 /* Create an object from a file. Attach some info-headers to it */
-obex_object_t *build_object_from_file(obex_t *handle, const char *localname, const char *remotename)
+obex_object_t *build_object_from_file(obex_t *obex, uint32_t conn, const char *localname, const char *remotename)
 {
 	obex_object_t *object = NULL;
 	uint8_t *ucname;
@@ -72,32 +73,34 @@ obex_object_t *build_object_from_file(obex_t *handle, const char *localname, con
 	/* Get filesize and modification-time */
 	size = get_fileinfo(localname, lastmod);
 
-	object = OBEX_ObjectNew(handle, OBEX_CMD_PUT);
+	object = OBEX_ObjectNew(obex, OBEX_CMD_PUT);
 	if(object == NULL)
 		return NULL;
 
+        if(conn != 0xffffffff)
+		(void) OBEX_ObjectAddHeader(obex, object, OBEX_HDR_CONNECTION, (obex_headerdata_t) conn, sizeof(uint32_t), OBEX_FL_FIT_ONE_PACKET);
+
 	ucname_len = strlen(remotename)*2 + 2;
 	ucname = malloc(ucname_len);
 	if(ucname == NULL) {
-		if(object != NULL)
-			(void) OBEX_ObjectDelete(handle, object);
+		(void) OBEX_ObjectDelete(obex, object);
 		return NULL;
 	}
 
 	ucname_len = OBEX_CharToUnicode(ucname, remotename, ucname_len);
 
-	(void ) OBEX_ObjectAddHeader(handle, object, OBEX_HDR_NAME, (obex_headerdata_t) (const uint8_t *) ucname, ucname_len, 0);
+	(void ) OBEX_ObjectAddHeader(obex, object, OBEX_HDR_NAME, (obex_headerdata_t) (const uint8_t *) ucname, ucname_len, 0);
 	free(ucname);
 
-	(void) OBEX_ObjectAddHeader(handle, object, OBEX_HDR_LENGTH, (obex_headerdata_t) (const uint32_t)size, sizeof(uint32_t), 0);
+	(void) OBEX_ObjectAddHeader(obex, object, OBEX_HDR_LENGTH, (obex_headerdata_t) (const uint32_t)size, sizeof(uint32_t), 0);
 
 #if 0
 	/* Win2k excpects this header to be in unicode. I suspect this in
 	   incorrect so this will have to wait until that's investigated */
-	OBEX_ObjectAddHeader(handle, object, OBEX_HDR_TIME, (obex_headerdata_t) (const uint8_t *) lastmod, strlen(lastmod)+1, 0);
+	OBEX_ObjectAddHeader(obex, object, OBEX_HDR_TIME, (obex_headerdata_t) (const uint8_t *) lastmod, strlen(lastmod)+1, 0);
 #endif
 		
-	(void) OBEX_ObjectAddHeader(handle, object, OBEX_HDR_BODY,
+	(void) OBEX_ObjectAddHeader(obex, object, OBEX_HDR_BODY,
 				(obex_headerdata_t) (const uint8_t *) NULL,
 				0, OBEX_FL_STREAM_START);
 
@@ -130,24 +133,29 @@ static int nameok(const char *name)
 /* Concatenate two pathnames. */
 /* The first path may be NULL. */
 /* The second path is always treated relative. */
-/* dest must have at least _POSIX_PATH_MAX + 1 chars. */
-static int pathcat(/*@unique@*/ char *dest, const char *path, const char *name)
+static int pathncat(/*@unique@*/ char *dest, const char *path, const char *name, size_t n)
 {
+	size_t len;
 	if(name == NULL)
-		return 1;
+		return -EINVAL;
 		
 	while(*name == '/')
 		name++;
 		
-	if((path == NULL) || (*path == '\0'))
-		strncpy(dest, name, _POSIX_PATH_MAX);
-	else {
-		strncpy(dest, path, _POSIX_PATH_MAX);
-		if (dest[strlen(dest)-1] != '/') {
-			dest[strlen(dest) + 1] = '\0';
-			dest[strlen(dest)] = '/';
+	if((path == NULL) || (*path == '\0')) {
+		strncpy(dest, name, n);
+		dest[n - 1] = '\0';
+	} else {
+		strncpy(dest, path, n);
+		dest[n - 1] = '\0';
+		len = strlen(dest);
+		if (len >= n - 1)
+			return -ENOMEM;
+		if (dest[len - 1] != '/') {
+			dest[len] = '/';
+			dest[len + 1] = '\0';
 		}
-		strncat(dest, name, _POSIX_PATH_MAX-strlen(dest));
+		strncat(dest, name, n - strlen(dest) - 1);
 	}
 
 	return 0;
@@ -156,7 +164,8 @@ static int pathcat(/*@unique@*/ char *dest, const char *path, const char *name)
 /* Open a file, but do some sanity-checking first. */
 int open_safe(const char *path, const char *name)
 {
-	char diskname[_POSIX_PATH_MAX + 1] = {0,};
+	char *diskname;
+	size_t maxlen;
 	int fd;
 
 	DEBUG(3, "%s() \n", __func__);
@@ -167,18 +176,26 @@ int open_safe(const char *path, const char *name)
 
 	/* TODO! Rename file if already exist. */
 
-	(void) pathcat(diskname, path, name);
+        maxlen = strlen(name) + 1;
+	if (path)
+        	maxlen += strlen(path);
+	diskname = malloc(maxlen);
+	if(!diskname)
+		return -1;
+	(void) pathncat(diskname, path, name, maxlen);
 
 	DEBUG(3, "%s() Creating file %s\n", __func__, diskname);
 
 	fd = open(diskname, O_RDWR | O_CREAT | O_TRUNC, DEFFILEMOD);
+	free(diskname);
 	return fd;
 }
 
 /* Go to a directory. Create if not exists and create is true. */
 int checkdir(const char *path, const char *dir, int create, int allowabs)
 {
-	char newpath[_POSIX_PATH_MAX + 1] = {0,};
+	char *newpath;
+	size_t maxlen;
 	struct stat statbuf;
 	int ret = -1;
 
@@ -187,18 +204,28 @@ int checkdir(const char *path, const char *dir, int create, int allowabs)
 			return -1;
 	}
 
-	(void) pathcat(newpath, path, dir);
+	if(!dir)
+		return 1;
+        maxlen = strlen(dir) + 1;
+	if (path)
+        	maxlen += strlen(path);
+	newpath = malloc(maxlen);
+	if(!newpath)
+		return -1;
+	(void) pathncat(newpath, path, dir, maxlen);
 
 	DEBUG(3, "%s() path = %s dir = %s, create = %d, allowabs = %d\n", __func__, path, dir, create, allowabs);
 	if(stat(newpath, &statbuf) == 0) {
 		/* If this directory aleady exist we are done */
 		if(S_ISDIR(statbuf.st_mode)) {
 			DEBUG(3, "%s() Using existing dir\n", __func__);
+			free(newpath);
 			return 1;
 		}
 		else  {
 			/* A non-directory with this name already exist. */
 			DEBUG(3, "%s() A non-dir called %s already exist\n", __func__, newpath);
+			free(newpath);
 			return -1;
 		}
 	}
@@ -214,6 +241,7 @@ int checkdir(const char *path, const char *dir, int create, int allowabs)
 		ret = -1;
 	}
 
+	free(newpath);
 	return ret;
 }
 	
